# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from colander import MappingSchema
from colander import Length
from colander import OneOf
from colander import Schema
from colander import SchemaNode
from colander import String
from deform.widget import RadioChoiceWidget
from deform.widget import TextInputWidget
from deform.widget import TextAreaWidget


class QA(MappingSchema):
    """Schema for the QA Assessment data."""

    work_needed_link = SchemaNode(
        String(),
        missing='',
        validator=Length(max=1000),
        widget=TextInputWidget(size=60),
        description="Link of work needed to make charm ideal")

    notes = SchemaNode(
        String(),
        missing='',
        widget=TextAreaWidget(rows=4, cols=60))


def get_qa_form_schema(score_questions):
    """We dynamically add the list of qa score questions to the form.

    These are defined in the collection db.qa.

    """
    schema = QA()
    choices = (
        ('1', 'Yes'),
        ('0', 'No'),
        ('', 'Unknown'),
    )
    validator = OneOf([x[0] for x in choices])
    for sq in score_questions:
        category = sq['name']
        category_description = sq['description']
        questions = sq['questions']

        category_schema = Schema(
            name=category,
            title=category_description)

        for question in questions:
            node = SchemaNode(
                String(),
                missing='',
                validator=validator,
                widget=RadioChoiceWidget(values=choices),
                name=question['id'],
                title=question['description'],
                description=question['extended_description'],
            )
            category_schema.add(node)

        schema.add(category_schema)

    return schema
