# Copyright 2014 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from deform import (
    Form,
    ValidationFailure,
)
from pyramid.httpexceptions import HTTPFound
from pyramid.view import view_config

from charmworld.forms.delete import get_schema
from charmworld.search import (
    BUNDLE,
    CHARM,
)
from charmworld.utils import (
    delete_bundle,
    delete_charm,
)
from charmworld.views.helpers import (
    find_charm,
    find_bundle,
    found,
)


def delete_item(item, kind, db, index_client):
    """Dispatch the deletion of the charm or bundle."""
    if kind == BUNDLE:
        delete_bundle(item, db, index_client)
    elif kind == CHARM:
        delete_charm(item, db, index_client)
    else:
        raise ValueError('Unknown entity type: {}'.format(kind))


def delete_confirm(request, item, kind, next_url):
    """Back end for deleting an 'item' which can be a charm or bundle."""
    form = Form(get_schema(kind), buttons=('submit', 'cancel'))
    form.css_class = 'form-horizontal well'

    if 'cancel' in request.POST:
        return HTTPFound(next_url)
    elif 'submit' in request.POST:
        # Delete the item.
        try:
            data = request.POST.items()
            valid_data = form.validate(data)
            if valid_data['delete_it']:
                delete_item(item, kind, request.db, request.index_client)
            # Send the user back to the next_url.
            return HTTPFound(next_url)
        except ValidationFailure as form:
            valid_data = {}
    else:
        valid_data = {'delete_it': False}
    return {
        'form': form.render(valid_data),
        'values': valid_data,
    }


@view_config(
    route_name='charm-delete',
    renderer='charmworld:templates/delete.pt',
    permission='edit',
)
@view_config(
    route_name='personal-charm-delete',
    renderer='charmworld:templates/delete.pt',
    permission='edit',
)
def charm_delete(request):
    """Back end for deleting a charm."""
    promulgated = 'owner' not in request.matchdict
    charm = find_charm(request, promulgated)
    found(charm)
    return delete_confirm(request, charm, 'charm', next_url='/')


@view_config(
    route_name='bundle-delete',
    renderer='charmworld:templates/delete.pt',
    permission='edit',
)
@view_config(
    route_name='personal-bundle-delete',
    renderer='charmworld:templates/delete.pt',
    permission='edit',
)
def bundle_delete(request):
    """Back end for deleting a bundle."""
    promulgated = 'owner' not in request.matchdict
    bundle = find_bundle(request, promulgated)
    # Assert that the bundle was found, generating a 404 if not.
    found(bundle)
    return delete_confirm(request, dict(bundle), BUNDLE, next_url='/')
