# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).
from pyelasticsearch.exceptions import ElasticHttpError
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render_to_response
from pyramid.view import view_config

from charmworld.search import (
    IndexNotReady,
    SearchServiceNotAvailable,
)
from charmworld.views import log


def do_search(request):
    text = request.params["search_text"]
    return request.index_client.search(text)


@view_config(
    route_name="search",
    renderer="charmworld:templates/search.pt")
def search(request):
    if not "search_text" in request.params:
        url = request.route_url("home")
        return HTTPFound(location=url)
    try:
        search = do_search(request)
    except (IndexNotReady, SearchServiceNotAvailable):
        response = render_to_response('../templates/search-not-ready.pt', {},
                                      request=request)
        response.status_code = 503
        return response
    except ElasticHttpError as exc:
        text = request.params["search_text"]
        log.warning(
            "User search error with search text='{}'".format(text))
        log.exception(exc)
        response = render_to_response(
            '../templates/search-terms-bad.pt',
            {'search_text': text},
            request=request)
        response.status_code = 400
        return response
    return dict((key, value) for key, value in search.items()
                if key != 'matches')
