import json
from webob import Response
from charmworld.models import DatedMetricSource
from charmworld.utils import (
    build_metric_key,
    utc_today,
)


def json_response(status, value, headers=[]):
    """Return a JSON API response.

    :param status: The HTTP status code to use.
    :param value: A json-serializable value to use as the body, or None for
        no body.
    """
    # If the value is an object, then it must be representable as a mapping.
    if value is None:
        body = ''
    else:
        while True:
            try:
                body = json.dumps(value, sort_keys=True, indent=2)
            except TypeError:
                # If the serialization wasn't possible, and the value wasn't
                # already a dictionary, convert it to one and try again.
                if not isinstance(value, dict):
                    value = dict(value)
                    continue
            break

    return Response(body,
                    headerlist=[
                        ('Content-Type', 'application/json'),
                        ('Access-Control-Allow-Origin', '*'),
                        ('Access-Control-Allow-Headers', 'X-Requested-With'),
                    ] + headers,
                    status_code=status)


class Statistics:
    """A template helper that wraps a DatedMetric instance."""

    def __init__(self, metric, day=None):
        day = day or utc_today()
        self.past_7_days = metric.get_range_total(7, day)
        self.past_30_days = metric.get_range_total(30, day)
        self.past_half_year = metric.get_range_total(365 / 2, day)
        self.total = metric.get_total()


def get_bundle_downloads(db, bundle_id):
    """Return a Statistics object for the bundle's downloads."""
    source = DatedMetricSource.from_db(db)
    metric_key = build_metric_key('deployments', bundle_id)
    metric = source.retrieve(metric_key, create=True)
    return Statistics(metric)
